/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMHTMLElement.idl"

/**
 * The nsIDOMHTMLTableCellElement interface is the interface to a
 * [X]HTML td element.
 *
 * This interface is trying to follow the DOM Level 2 HTML specification:
 * http://www.w3.org/TR/DOM-Level-2-HTML/
 *
 * with changes from the work-in-progress WHATWG HTML specification:
 * http://www.whatwg.org/specs/web-apps/current-work/
 */

[scriptable, uuid(1fb22137-c231-4eae-9ca4-557bb0852d89)]
interface nsIDOMHTMLTableCellElement : nsIDOMHTMLElement
{
  readonly attribute long             cellIndex;
           attribute DOMString        abbr;
           attribute DOMString        align;
           attribute DOMString        axis;
           attribute DOMString        bgColor;
           attribute DOMString        ch;
           attribute DOMString        chOff;
           attribute long             colSpan;
           attribute DOMString        headers;
           attribute DOMString        height;
           attribute boolean          noWrap;
           attribute long             rowSpan;
           attribute DOMString        scope;
           attribute DOMString        vAlign;
           attribute DOMString        width;
};
