/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

/*

  This file contains some constants for the cookie accept dialog
  
  */

[scriptable, uuid(3F2F0D2C-BDEA-4B5A-AFC6-FCF18F66B97E)]
interface nsICookieAcceptDialog: nsISupports {

  const short ACCEPT_COOKIE=0;
  const short REMEMBER_DECISION=1;
  const short HOSTNAME=2;
  const short COOKIESFROMHOST=3;
  const short CHANGINGCOOKIE=4;
};
