/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMXULDescriptionElement.idl"

[scriptable, uuid(8b67286b-bf86-4808-8a68-ecdbc7d9877b)]
interface nsIDOMXULLabelElement : nsIDOMXULDescriptionElement {
  attribute DOMString accessKey;
  attribute DOMString control;
};

