/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

interface nsIPK11Token;
interface nsIEnumerator;

/**
 * The PK11 Token Database provides access to the PK11 modules
 * that are installed, and the tokens that are available.
 * Interfaces: nsIPK11TokenDB
 * Threading: ??
 */
%{C++
#define NS_PK11TOKENDB_CONTRACTID "@mozilla.org/security/pk11tokendb;1"
%}

/**
 * nsIPK11TokenDB - Manages PK11 Tokens
 */
[scriptable, uuid(4ee28c82-1dd2-11b2-aabf-bb4017abe395)]
interface nsIPK11TokenDB : nsISupports
{
  /*
   * Get the internal key database token
   */
  nsIPK11Token getInternalKeyToken();

  /*
   * Find a token by name
   */
  nsIPK11Token findTokenByName(in wstring tokenName);

  /*
   * List all tokens
   */
  nsIEnumerator listTokens();

};

